package test;

import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.*;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = {RedisConfig.class})
public class RedisTest {


    @Autowired
    private RedisTemplate<String, Category> categoryTemplate;

    @Test
    public void testCategoryTemplate() {
        String key = "category";
        categoryTemplate.delete(key);

        Category category = new Category(0, "Aa");
        categoryTemplate.opsForSet().add(key, category);

        Set<Category> set = categoryTemplate.opsForSet().members(key);

        Assert.assertTrue(set != null && set.contains(category));
    }

    @Autowired
    private RedisTemplate<String, Student> studentTemplate;

    @Test
    public void testStudentTemplate() {
        String key = "student";
        studentTemplate.delete(key);

        Student student = new Student(0, "Aa");
        studentTemplate.opsForSet().add(key, student);

        Set<Student> set = studentTemplate.opsForSet().members(key);

        Assert.assertTrue(set != null && set.contains(student));
    }

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Test
    public void testStringRedisTemplate() throws Exception {

        String key = "stringRedisTemplate";
        stringRedisTemplate.delete(key);

        Student student = new Student(1, "Bb");

        ObjectMapper objectMapper = new ObjectMapper();

        String studentString = objectMapper.writeValueAsString(student);
        //写入、读取单个值
        stringRedisTemplate.opsForList().leftPush(key, studentString);
        String result = stringRedisTemplate.opsForList().leftPop(key);
        Assert.assertEquals(result, studentString);

        List<Student> studentList = new ArrayList<>();
        Map<String, Student> studentMap = new HashMap<>();
        for (int i = 0; i < 5; i++) {
            Student temp = new Student(i, "name" + i);
            studentList.add(temp);
            studentMap.put("" + i, temp);
        }

        //存取ArrayList<Student>
        stringRedisTemplate.delete("studentList");
        stringRedisTemplate.opsForValue().set("studentList", objectMapper.writeValueAsString(studentList));
        String studentListString = stringRedisTemplate.opsForValue().get("studentList");
        JavaType javaType1 = objectMapper.getTypeFactory().constructParametricType(ArrayList.class, Student.class);
        List<Student> studentList1 = objectMapper.readValue(studentListString, javaType1);

        Assert.assertEquals(studentList, studentList1);

        //存取Map
        stringRedisTemplate.delete("studentMap");
        stringRedisTemplate.opsForValue().set("studentMap", objectMapper.writeValueAsString(studentMap));
        JavaType javaType2 = objectMapper.getTypeFactory().constructParametricType(HashMap.class, String.class, Student.class);
        Map<String, Student> resultMap = objectMapper.readValue(stringRedisTemplate.opsForValue().get("studentMap"), javaType2);

        Assert.assertEquals(studentMap.entrySet(), resultMap.entrySet());
    }

    @Autowired
    private RedisTemplate<String, Object> objectRedisTemplate;

    @Test
    public void testObjectTemplate() {
        String key = "object";
        objectRedisTemplate.delete(key);

        //存取单个实例
        Student student = new Student(1, "Bb");
        objectRedisTemplate.opsForValue().set(key, student);
        Object result = objectRedisTemplate.opsForValue().get(key);
        Assert.assertEquals(student, result);

        //存取集合类
        List<Student> list = new ArrayList<>();
        for (int i = 0; i < 3; i++) {
            list.add(new Student(i, "name" + i));
        }

        objectRedisTemplate.delete("list");
        objectRedisTemplate.opsForList().leftPushAll("list", list);
        List<Object> resultList = objectRedisTemplate.opsForList().range("list", 0, -1);  //取出来的数据外面包了一层List
        if (resultList != null && ((List) resultList).size() > 0 && resultList.get(0) instanceof List) {
            resultList = (List) resultList.get(0);
            Assert.assertEquals(list, resultList);
        } else {
            Assert.fail();
        }
    }

    public static class Student {

        public Student(int id, String name) {
            this.id = id;
            this.name = name;
        }

        public Student() {
        }

        private int id;

        private String name;

        public int getId() {
            return id;
        }

        public void setId(int id) {
            this.id = id;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            Student student = (Student) o;
            return id == student.id &&
                    name.equals(student.name);
        }

        @Override
        public int hashCode() {
            return Objects.hash(id, name);
        }

        @Override
        public String toString() {
            return super.toString();
        }
    }
}
